# -*- coding: utf-8 -*-
# Copyright (C) 2006 Ashwin Phatak
# Copyright (C) 2007 Dave Gynn
import re

from trac.core import *    
from trac.ticket import TicketSystem, Ticket, Milestone
from trac.ticket.report import ReportModule
from trac.web.api import ITemplateStreamFilter
from trac.web.chrome import ITemplateProvider, Chrome
from trac.web.main import IRequestFilter
from trac.util import partition
from trac.util.translation import _

from genshi.filters.transform import Transformer
from genshi.builder import tag
from pkg_resources import resource_filename


__all__ = ['BatchModifyModule']



def group_milestones(milestones, include_completed):
    """Group milestones into "open with due date", "open with no due date",
    and possibly "completed". Return a list of (label, milestones) tuples."""
    def category(m):
        return m.is_completed and 1 or m.due and 2 or 3
    open_due_milestones, open_not_due_milestones, \
        closed_milestones = partition([(m, category(m))
            for m in milestones], (2, 3, 1))
    groups = [
        (_('Open (by due date)'), open_due_milestones),
        (_('Open (no due date)'), open_not_due_milestones),
    ]
    if include_completed:
        groups.append((_('Closed'), closed_milestones))
    return groups


class BatchModifyModule(Component):
    implements(ITemplateProvider, IRequestFilter, \
    ITemplateStreamFilter)

    # ITemplateProvider methods
    def get_htdocs_dirs(self):
        return []

    def get_templates_dirs(self):

        return [resource_filename(__name__, 'templates')]








    # IRequestFilter methods
    def pre_process_request(self, req, handler):
        """Look for QueryHandler posts and hijack them"""
        if isinstance(handler, ReportModule):
            if req.method=='POST' and req.args.get('batchmod'):                
                self._batch_modify(req)
                #this prevent a problem with templates
                #after pressing the submit button
                req.redirect(req.base_path + req.path_info)
        return handler

    def post_process_request(self, req, template, data, content_type):
        """No-op"""
        return (template, data, content_type)
    
    # Internal methods 
    def _batch_modify(self, req):
        """Takes the POST request and proccess all the actions"""
        tickets = req.session['query_tickets'].split(' ')
        comment = req.args.get('bmod_value_comment', '') 
        values = {} 
        # TODO: improve validation and better handle advanced statuses
        for field in TicketSystem(self.env).get_ticket_fields():
            name = field['name']
            if name == 'resolution' and req.args.has_key('bmod_flag_' + name):
                values['status'] = 'closed'
            if name not in ('summary', 'reporter', 'description'):
                if req.args.has_key('bmod_flag_' + name):
                    values[name] = req.args.get('bmod_value_' + name)
        selectedTickets = req.args.get('selectedTickets')
        selectedTickets = isinstance(selectedTickets, list) and selectedTickets or selectedTickets.split(',')
        if str(selectedTickets[0]) == '':
            raise TracError, 'No Tickets selected'
        def _toNumbers(st):
            """Sanitize the input tickets"""
            exp = '(?P<val>[0-9]+)'
            match = re.search(exp, st)
            return str(match.groupdict().get('val'))
        selectedTickets = map(_toNumbers, selectedTickets)
        for id in selectedTickets:
            if id in tickets:
                t = Ticket(self.env, id)
                t.populate(values)
                t.save_changes(req.authname, comment)

                # TODO: Send email notifications - copied from ticket.web_ui
                #try:
                #    tn = TicketNotifyEmail(self.env)
                #    tn.notify(ticket, newticket=False, modtime=now)
                #except Exception, e:
                #    self.log.exception("Failure sending notification on change to "
                #                       "ticket #%s: %s" % (ticket.id, e))
    
                # TODO: deal with actions and side effects - copied from ticket.web_ui
                #for controller in self._get_action_controllers(req, ticket,
                #                                               action):
                #    controller.apply_action_side_effects(req, ticket, action)

    # ITemplateStreamFilter methods
    def filter_stream(self, req, method, filename, stream, formdata):
        """Adds BatchModify form to the query page"""

        if (filename == 'report_view.html') and (req.path_info != '/report') \
        and (req.perm.has_permission('TICKET_ADMIN') ):
            stream |= Transformer('.//thead//tr') \
                    .prepend(tag.th('')(tag.input(type='checkbox',   \
                    name='bmod_toggleGroup')))
            return stream | Transformer('//div[@class="buttons"]'). \
                                after(self._generate_form(req, formdata) )
        return stream

    
    def _generate_form(self, req, data):
        batchFormData = dict(data)
        batchFormData['actionUri']= req.base_path + req.path_info
        fields = []
        for field in TicketSystem(self.env).get_ticket_fields():
            if field['name'] == 'milestone':
                milestones = [Milestone(self.env, opt)
                    for opt in field['options']]
                milestones = [m for m in milestones
                        if 'MILESTONE_VIEW' in req.perm(m.resource)]
                groups = group_milestones(milestones, 1)
                field['options'] = []
                field['optgroups'] = [
                    {'label': label, 'options': [m.name for m in milestones]}
                    for (label, milestones) in groups]


            if field['name'] not in ('summary', 'reporter', 'description'):
                fields.append(field)
        batchFormData['fields']=fields
        stream = Chrome(self.env).render_template(req, 'batchmod.html',
              batchFormData, fragment=True)
        return stream.select('//form[@id="batchmod-form"]')



